package com.smardec.demo;

import com.smardec.demo.tools.LibraryManager;
import com.smardec.j2native.*;

import javax.swing.*;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;


public class J2NMain{
    public static final String KERNEL_NAME = "kernel32";
    public static final String USER_NAME = "user32";
    public static final String GDI_NAME = "gdi32";
    private static J2NMain instance;
    private static Library KERNEL;
    private static Library USER;
    private static Library GDI;

    private MainFrame mainFrame;

    public J2NMain(){
        mainFrame = new MainFrame();
        mainFrame.addWindowListener(new WindowHandler());
        mainFrame.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
    }

    public void messageBox(String caption, String text, int flags){
        J2NMain.showMessageBox(caption, text, flags);
    }

    public void messageBeep(int tone){
        Function messageBeep = USER.getFunction("MessageBeep");
        messageBeep.invoke(null, new Int32(tone));
    }


    protected void run(){
        mainFrame.show();
    }

    public static J2NMain getInstance(){
        if(instance == null){
            instance = new J2NMain();
        }
        return instance;
    }

    public static void main(String[] args){
        Library.ensureNativeCodeLoaded();
        initLibManager();
        if(!ensureOneInstance()){
            System.err.println("Only one instance at the same time allowed.");
            resetLibManager();
            System.exit(1);
        }

        getInstance().run();
    }

    private static boolean ensureOneInstance(){
        Pointer mutexName = new Pointer(new AnsiString("com.smardec.demo.J2NMain"));
        UInt32 desiredAcces = new UInt32(0x1F0001);
        Bool inheritHandle = new Bool(false);
        Pointer.Void mutexHandle = new Pointer.Void();
        Function openMutex = KERNEL.getFunction("OpenMutexA");
        openMutex.invoke(mutexHandle, desiredAcces, inheritHandle, mutexName);
        if(!mutexHandle.isNull()){
            // Mutex exists - one instance is already running
            return false;
        }

        // Not yet running - lock by creating mutex
        Pointer.Void mutexAttributes = new Pointer.Void(0);
        Bool initialOwner = new Bool(false);
        Function createMutex = KERNEL.getFunction("CreateMutexA");
        createMutex.invoke(mutexHandle, mutexAttributes, initialOwner, mutexName);
        if(mutexHandle.isNull()){
            throw new RuntimeException("Mutex creation failed, last error = " + getLastError(true));
        }
        return true;
    }


    private static long getLastError(boolean clear){
        UInt32 r = new UInt32();
        Function getLE = KERNEL.getFunction("GetLastError");
        getLE.invoke(r);
        long errCode = r.longValue();
        if(clear){
            UInt32 zero = new UInt32(0);
            Function setLE = KERNEL.getFunction("SetLastError");
            setLE.invoke(null, zero);
        }
        return errCode;
    }


    private static void showMessageBox(String caption, String message, int flags){
        Pointer.Void hWnd = new Pointer.Void();
        Pointer pText = new Pointer(new AnsiString(message));
        Pointer pCaption = new Pointer(new AnsiString(caption));
        UInt uFlags = new UInt(flags);
        // For functions taking strings as parameters there are two versions: ANSI and Unicode
        // We chose ANSI here and hence function name ends with A
        // It should be noted that there is no plain MessageBox function exported
        Function findWindow = USER.getFunction("FindWindowA");
        findWindow.invoke(hWnd, new Pointer.Void(), new Pointer(new AnsiString("J2Native Demo")));

        Function function = USER.getFunction("MessageBoxA");
        function.invoke(null, hWnd, pText, pCaption, uFlags);
    }

    private static void resetLibManager(){
        LibraryManager.getInstance().reset();
    }

    private static void initLibManager(){
        LibraryManager libManager = LibraryManager.getInstance();
        Library library = new Library(KERNEL_NAME);
        libManager.registerLibrary(library);
        library = new Library(USER_NAME);
        libManager.registerLibrary(library);
        library = new Library(GDI_NAME);
        libManager.registerLibrary(library);

        KERNEL = libManager.getLibrary(KERNEL_NAME);
        USER = libManager.getLibrary(USER_NAME);
        GDI = libManager.getLibrary(GDI_NAME);
    }

    private class WindowHandler extends WindowAdapter{
        public void windowClosing(WindowEvent e){
            J2NMain.resetLibManager();
            System.exit(0);
        }
    }
}
